# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
print(__file__)

import bpy, bpy_extras
import io_export_i3d.i3d_export
import io_export_i3d.dcc as dcc


def _MT_draw_I3D_Menu( self, context ):
    self.layout.menu( _PT_I3D_PanelExport.bl_idname )

#-------------------------------------------------------------------------------
#   "I3D Exporter" I3D Menu Item
#-------------------------------------------------------------------------------
class _MT_I3D_MenuExport( bpy.types.Operator ):
    bl_label = "I3D Exporter"
    bl_idname = "i3d.menu_export"

    def execute( self, context ):
        try:                  
            
            #bpy.utils.register_class( _PT_I3D_PanelExport )            
            #bpy.utils.register_class( _PT_I3D_PanelExport_ButtonClose )
            #bpy.utils.register_class( _PT_I3D_PanelExport_ButtonExport )
            #bpy.utils.register_class( _PT_I3D_PanelExport_ButtonAttr )

            classes = (_PT_I3D_PanelExport,  _PT_I3D_PanelExport_ButtonClose, _PT_I3D_PanelExport_ButtonExport, _PT_I3D_PanelExport_ButtonAttr,)

            register, unregister = bpy.utils.register_classes_factory(classes)

            


        except: 
            print("failes")           
            pass
        return {'FINISHED'}
 




#-------------------------------------------------------------------------------
#   File -> Export     menu bottom left after export push
#-------------------------------------------------------------------------------

class I3D_FileExport( bpy.types.Operator, bpy_extras.io_utils.ExportHelper , bpy.types.Menu):
    bl_idname           = "i3d.file_export"
    bl_label            = "Export I3D"
    bl_options          = {'PRESET'}
    filename_ext        = ".i3d"
    I3D_exportSelected  : bpy.props.BoolProperty   ( name = "Export Selected",  default = False )

    def __init__(self):
        self.filepath      = bpy.context.scene.I3D_UIexportSettings.I3D_exportFileLocation
        
    def draw( self, context ):
        layout = self.layout
        #-----------------------------------------
        # "Export Selected" box
        box = layout.box()
        row = box.row()
        row.prop( self,  "I3D_exportSelected" )
        #-----------------------------------------
        # "Export Options" box
        box = layout.box()
        row = box.row()
        # expand button for "Export Options"
        row.prop(   context.scene.I3D_UIexportSettings, 
                    "UI_exportOptions", 
                    icon='TRIA_DOWN' if context.scene.I3D_UIexportSettings.UI_exportOptions else 'TRIA_RIGHT', 
                    icon_only = True, 
                    emboss = False )
        # expanded view
        if context.scene.I3D_UIexportSettings.UI_exportOptions:
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportIK"        )            
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportAnimation" )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportShapes"    )
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportNurbsCurves" )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportLights"      )
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportCameras"     )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportParticleSystems" )
            row.prop( context.scene.I3D_UIexportSettings,  "I3D_exportUserAttributes"  )
        #-----------------------------------------
        # "Shape Export Subparts" box
        box = layout.box()
        row = box.row()
        # expand button for "Shape Export Subparts"
        row.prop(   context.scene.I3D_UIexportSettings, 
                    "UI_shapeExportSubparts", 
                    icon='TRIA_DOWN' if context.scene.I3D_UIexportSettings.UI_shapeExportSubparts else 'TRIA_RIGHT', 
                    icon_only = True, 
                    emboss = False )
        if context.scene.I3D_UIexportSettings.UI_shapeExportSubparts:
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportNormals"     )
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportTexCoords"   )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportColors"      )
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportSkinWeigths" )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportMergeGroups")
        #-----------------------------------------
        # "Miscellaneous" box
        box = layout.box()
        row = box.row()
        # expand button for "Miscellaneous"
        row.prop(   context.scene.I3D_UIexportSettings, 
                    "UI_miscellaneous", 
                    icon='TRIA_DOWN' if context.scene.I3D_UIexportSettings.UI_miscellaneous else 'TRIA_RIGHT', 
                    icon_only = True, 
                    emboss = False )
        if context.scene.I3D_UIexportSettings.UI_miscellaneous:
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportVerbose"       )
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportRelativePaths" )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportApplyModifiers"  )
            row = box.row()
            row.prop( context.scene.I3D_UIexportSettings, "I3D_exportAxisOrientations"  )
        #-----------------------------------------
        
    def execute(self, context):
        bpy.context.scene.I3D_UIexportSettings.I3D_exportUseSoftwareFileName   = False
        bpy.context.scene.I3D_UIexportSettings.I3D_exportFileLocation         = self.filepath
        if ( self.I3D_exportSelected ):
            io_export_i3d.i3d_export.I3DExportSelected()
        else:
            io_export_i3d.i3d_export.I3DExportAll()
        return {'FINISHED'}
    

    
        
def fileExportMenuItem(self, context):
    self.layout.operator( I3D_FileExport.bl_idname, text ="GIANTS I3D (.i3d)" )
    


class _PT_I3D_PanelExport_ButtonAttr( bpy.types.Operator ):
    bl_idname  = "i3d.panel_export_attr"
    bl_label   = "Attributes"
    state      : bpy.props.IntProperty()
    
    def execute( self, context ):
        if   1 == self.state:
            dcc.I3DLoadObjectAttributes()
        elif 2 == self.state:
            dcc.I3DSaveObjectAttributes()
        elif 3 == self.state:
            dcc.I3DRemoveObjectAttributes()
        elif 4 == self.state:
            dcc.I3DApplySelectedAttributes()
        elif 5 == self.state:
            dcc.I3DRemoveSelectedAttributes()
        return {'FINISHED'}  

class _PT_I3D_PanelExport_ButtonExport( bpy.types.Operator ):
    bl_idname  = "i3d.panel_export_do"
    bl_label   = "Export"
    state      : bpy.props.IntProperty()
    
    def execute( self, context ):
        if   1 == self.state:
            io_export_i3d.i3d_export.I3DExportAll()
        elif 2 == self.state:
            io_export_i3d.i3d_export.I3DExportSelected()
        return {'FINISHED'}  
        
class _PT_I3D_PanelExport_ButtonClose( bpy.types.Operator ):
    bl_idname  = "i3d.panel_export_close"
    bl_label   = "Close"

    def execute( self, context ):
        bpy.utils.unregister_class( _PT_I3D_PanelExport )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonClose )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonExport )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonAttr )
        return {'FINISHED'}



#-------------------------------------------------------------------------------
#   Register
#-------------------------------------------------------------------------------
def register():
   
    # --------------------------------------------------------------------------
    bpy.utils.register_class( _MT_I3D_MenuExport )
    # --------------------------------------------------------------------------
    bpy.utils.register_class( I3D_FileExport )    
    
    bpy.types.TOPBAR_MT_file_export.append( fileExportMenuItem )
    #bpy.types.ADDON_MT_I3dMenu.append( _MT_draw_I3D_Menu )
    
    
    


    
    

    
    

def unregister():
    
    bpy.utils.unregister_class( _MT_I3D_MenuExport )
   
    bpy.types.ADDON_MT_I3dMenu.remove( _MT_draw_I3D_Menu )
    # --------------------------------------------------------------------------
    try:
    
        bpy.utils.unregister_class( _PT_I3D_PanelExport )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonClose )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonExport )
        bpy.utils.unregister_class( _PT_I3D_PanelExport_ButtonAttr )
        
        
    except:
        pass
    # --------------------------------------------------------------------------
    bpy.utils.unregister_class( I3D_FileExport )
    
    bpy.types.TOPBAR_MT_file_export.remove( fileExportMenuItem )
    
    
    
    

if __name__ == "__main__":
    register()
#-------------------------------------------------------------------------------